@@ -25,17 +25,31 @@ import httplib2
 httplib2.RETRIES = 1
 
 import biosignalml.rdf as rdf
-from biosignalml.transports import BlockType, SignalData
-from biosignalml.transports import WebStreamReader, WebStreamWriter, StreamException
+from biosignalml.rdf.sparqlstore import SparqlUpdateStore
+from biosignalml.repository import BSMLUpdateStore
 
 
-def web_sockets_uri(uri):
-#========================
-  if uri.startswith('http'): return uri.replace('http', 'ws', 1)
-  else:                      return uri
+class RemoteSparqlStore(SparqlUpdateStore):
+#==========================================
+  """
+  Connect to SPARQL endpoints on a BioSignalML repository.
 
-class RemoteRepository(object):
-#==============================
+  """
+  ENDPOINTS = [ '/sparql/update/', '/sparql/graph/' ] #: Order is UPDATE, GRAPH
+
+  def __init__(self, uri, access_key):
+  #-----------------------------------
+    super(RemoteSparqlStore, self).__init__(uri)
+    self._access_key = access_key
+
+  def http_request(self, endpoint, method, body=None, headers=None):
+  #-----------------------------------------------------------------
+    if self._access_key is not None: headers['Cookie'] = 'access=%s' % self._access_key
+    return self._request(endpoint, method, body, headers)
+
+
+class RemoteRepository(BSMLUpdateStore):
+#=======================================
   '''
   A connection to a repository for both metadata and data.
   '''
@@ -44,9 +58,9 @@ class RemoteRepository(object):
   #----------------------------------------------------------------------------
     self.uri = uri
     self._access_key = access_key
+    super(RemoteRepository, self).__init__(uri, RemoteSparqlStore(uri, access_key))
     self._md_uri = uri + md_endpoint if md_endpoint is not None else ''
     self._sd_uri = uri + sd_endpoint if sd_endpoint is not None else ''
-    self._http = httplib2.Http(timeout=20)
     self.metadata = self.get_metadata(uri)
 
   def close(self):
@@ -68,7 +82,8 @@ class RemoteRepository(object):
     if self._access_key is not None: headers['Cookie'] = 'access=%s' % self._access_key
     endpoint = self._md_uri + str(uri)
     try:
-      response, content = self._http.request(endpoint, body=metadata, method=method, headers=headers)
+      http = httplib2.Http(timeout=20)
+      response, content = http.request(endpoint, body=metadata, method=method, headers=headers)
     except socket.error, msg:
       raise Exception("Cannot connect to repository: %s" % endpoint)
     if   response.status == 401: raise Exception("Unauthorised")
@@ -83,46 +98,6 @@ class RemoteRepository(object):
   #----------------------------------------------------------------
     return self._send_metadata('POST', uri, metadata, format)
 
-  def get_data(self, uri, **kwds):
-  #-------------------------------
-    """ Gets :class:`~biosignalml.data.DataSegment`\s from the remote repository. """
-    '''
-    maxsize
-    start
-    duration
-    offset
-    count
-    dtype
-    '''
-    kwds['access_key'] = self._access_key
-    reader = WebStreamReader(web_sockets_uri(self._sd_uri+uri), uri, **kwds)
-    for block in reader:
-      if block.type == BlockType.DATA: yield block.signaldata()
-    reader.join()
-
-  def put_data(self, uri, timeseries):
-  #-----------------------------------
-    stream = None
-    try:
-      stream = WebStreamWriter(web_sockets_uri(self._sd_uri+uri), access_key=self._access_key)
-      MAXPOINTS = 50000   ##### TESTING    (200K bytes if double precision)
-      params = { }
-      if hasattr(timeseries, 'rate'): params['rate'] = timeseries.rate
-      pos = 0
-      count = len(timeseries)
-      while count > 0:
-        blen = min(count, MAXPOINTS)
-        if hasattr(timeseries, 'clock'):
-          params['clock'] = timeseries.clock[pos:pos+blen]
-        stream.write_signal_data(SignalData(uri, timeseries.time[pos], timeseries.data[pos:pos+blen], **params))
-        pos += blen
-        count -= blen
-    except Exception, msg:
-      logging.error('Error in stream: %s', msg)
-      raise
-    finally:
-      if stream: stream.close()
-
 
 if __name__ == "__main__":
 #=========================
