@@ -366,7 +366,7 @@ class Graph(librdf.Model):
                                                        storage_name='hashes',
                                                        options_string="hash-type='memory'"))
     if isinstance(uri, Node): uri = uri.uri
-    if uri and not isinstance(uri, Uri): uri = Uri(uri)
+    if uri and not isinstance(uri, Uri): uri = Uri(str(uri))
     self.uri = uri
 
   @classmethod
@@ -398,7 +398,7 @@ class Graph(librdf.Model):
     :rtype: A :class:`Graph`
     """
     self = cls(uri)
-    self.parse_string(string, format, uri)
+    self.parse_string(string, format, str(uri))
     return self
 
   def __str__(self):
@@ -587,6 +587,16 @@ class Graph(librdf.Model):
     '''
     for v in self.get_objects(s, p): yield v.as_string()
 
+  def set_subject_property(self, s, p, o):
+  #---------------------------------------
+    """
+    Append the statement (s, p, o) to the graph after first removing
+    all statements with subject `s` and predicate `p`.
+    """
+    for stmt in self.find_statements(Statement(s, s, None)):
+      del self[stmt]
+    self.append(Statement(s, p, o))
+
   def query(self, sparql):
   #-----------------------
     '''
