@@ -63,40 +63,41 @@ class StoreException(Exception):
 #===============================
   pass
 
+
 class SparqlStore(object):
 #=========================
   """
   A SPARQL endpoint to a RDF store.
 
   :param href: The URL of the store.
-  :param endpoints: A list with the relative paths to Query,
-                    Update, and Graph endpoints.
-  """
 
-  ENDPOINTS = [ '/sparql/', '/update/', '/data/' ] #: Order is QUERY, UPDATE, GRAPH
+  """
 
   def __init__(self, href):
   #------------------------
     self._href = href
-    self._http = httplib2.Http(timeout=20)
 
   def _request(self, endpoint, method, body=None, headers=None):
   #-------------------------------------------------------------
     endpoint = self._href + endpoint
     try:
-      response, content = self._http.request(endpoint, body=body, method=method, headers=headers)
+      http = httplib2.Http(timeout=20)
+      response, content = http.request(endpoint, body=body, method=method, headers=headers)
     except socket.error, msg:
       raise StoreException("Cannot connect to SPARQL endpoint: %s" % endpoint)
     if response.status not in [200, 201]:
-      raise Exception(content)
+      raise StoreException('SPARQL request HTTP error %d: %s' % (response.status, response.reason))
     return content
 
+  def http_request(self, endpoint, method, body=None, headers=None):
+  #-----------------------------------------------------------------
+    return self._request(endpoint, method, body, headers)
+
   @staticmethod
   def map_prefixes(prefixes):
   #--------------------------
     return '\n'.join(['PREFIX %s: <%s>' % kv for kv in prefixes.iteritems()] + ['']) if prefixes else ''
 
-
   def query(self, sparql, format=rdf.Format.RDFXML, prefixes=None):
   #----------------------------------------------------------------
     """
@@ -104,10 +105,10 @@ class SparqlStore(object):
     """
     #logging.debug('SPARQL %s: %s', format, sparql)
     try:
-      return self._request(self.ENDPOINTS[0], 'POST',
-                           body=urllib.urlencode({'query': self.map_prefixes(prefixes) + sparql}),
-                           headers={'Content-type': 'application/x-www-form-urlencoded',
-                                    'Accept': rdf.Format.mimetype(format)} )
+      return self.http_request('/sparql/', 'POST',
+                               body=urllib.urlencode({'query': self.map_prefixes(prefixes) + sparql}),
+                               headers={'Content-type': 'application/x-www-form-urlencoded',
+                                        'Accept': rdf.Format.mimetype(format)} )
     except Exception, msg:
       logging.error('SPARQL: %s, %s', msg, self.map_prefixes(prefixes) + sparql)
       raise
@@ -177,6 +178,18 @@ class SparqlStore(object):
       "{ <%(uri)s> ?op ?o . optional { ?o a ?ot } } union { ?s ?sp <%(uri)s> . optional { ?s a ?st } }",
       params=dict(uri=uri), graph=graph, format=format)
 
+
+class SparqlUpdateStore(SparqlStore):
+#====================================
+  """
+  Connect to a SPARQL 1.1 update endpoint to a RDF store.
+
+  """
+
+  ENDPOINTS = [ '/update/', '/data/' ] #: Order is UPDATE, GRAPH
+  UPDATE_PARAMETER = 'update'
+  GRAPH_PARAMETER = 'graph'
+
   def update(self, sparql, prefixes=None):
   #---------------------------------------
     """
@@ -184,9 +197,9 @@ class SparqlStore(object):
     """
     ##logging.debug('UPD: %s', sparql)
     try:
-      return self._request(self.ENDPOINTS[1], 'POST',
-                           body=urllib.urlencode({self.UPDATE_PARAMETER: sparql + self.map_prefixes(prefixes)}),
-                           headers={'Content-type': 'application/x-www-form-urlencoded'})
+      return self.http_request(self.ENDPOINTS[0], 'POST',
+                               body=urllib.urlencode({self.UPDATE_PARAMETER: sparql + self.map_prefixes(prefixes)}),
+                               headers={'Content-type': 'application/x-www-form-urlencoded'})
     except Exception, msg:
       logging.error('SPARQL: %s, %s', msg, sparql + self.map_prefixes(prefixes))
       raise
@@ -240,29 +253,25 @@ class SparqlStore(object):
 
   def extend_graph(self, graph, statements, format=rdf.Format.RDFXML):
   #-------------------------------------------------------------------
-    '''
-    Extend an existing graph, creating one if not present.
-    '''
-    raise NotImplemented("SparqlStore.extend_graph()")
+    self.http_request(self.ENDPOINTS[1], 'POST',
+                      body=urllib.urlencode({'data': statements,
+                                             'graph': str(graph),
+                                             'mime-type': rdf.Format.mimetype(format),
+                                            }),
+                      headers={'Content-type': 'application/x-www-form-urlencoded'})
 
   def replace_graph(self, graph, statements, format=rdf.Format.RDFXML):
   #--------------------------------------------------------------------
-    '''
-    Replace an existing graph, creating one if not present.
-    '''
-    raise NotImplemented("SparqlStore.replace_graph()")
+    self.http_request(self.ENDPOINTS[1] + "?%s=%s" % (self.GRAPH_PARAMETER, graph), 'PUT',
+                      body=statements, headers={'Content-Type': rdf.Format.mimetype(format)})
 
   def delete_graph(self, graph):
   #-----------------------------
-    '''
-    Delete an existing graph from the store.
-    '''
-    raise NotImplemented("SparqlStore.delete_graph()")
+    self.http_request(self.ENDPOINTS[1] + "?%s=%s" % (self.GRAPH_PARAMETER, graph), 'DELETE')
 
 
-
-class Virtuoso(SparqlStore):
-#===========================
+class Virtuoso(SparqlUpdateStore):
+#=================================
   """
   Specifics for a Virtuoso SPARQL endpoint.
 
@@ -288,50 +297,20 @@ class Virtuoso(SparqlStore):
 
   """
 
-  ENDPOINTS = [ '/sparql/', '/sparql/', '/sparql-graph-crud/' ]
+  ENDPOINTS = [ '/sparql/', '/sparql-graph-crud/' ]
   UPDATE_PARAMETER = 'query'
+  GRAPH_PARAMETER = 'graph-uri'
 
 
   def extend_graph(self, graph, statements, format=rdf.Format.RDFXML):
   #-------------------------------------------------------------------
-    self._request(self.ENDPOINTS[2] + "?graph-uri=%s" % graph, 'POST',
-                  body=statements, headers={'Content-Type': rdf.Format.mimetype(format)})
-
-  def replace_graph(self, graph, statements, format=rdf.Format.RDFXML):
-  #--------------------------------------------------------------------
-    self._request(self.ENDPOINTS[2] + "?graph-uri=%s" % graph, 'PUT',
-                  body=statements, headers={'Content-Type': rdf.Format.mimetype(format)})
-
-  def delete_graph(self, graph):
-  #-----------------------------
-    self._request(self.ENDPOINTS[2] + "?graph-uri=%s" % graph, 'DELETE')
+    self.http_request(self.ENDPOINTS[1] + "?%s=%s" % (self.GRAPH_PARAMETER, graph), 'POST',
+                      body=statements, headers={'Content-Type': rdf.Format.mimetype(format)})
 
 
 
-def FourStore(SparqlStore):
-#==========================
-
-  ENDPOINTS = [ '/sparql/', '/update/', '/data/' ]
-  UPDATE_PARAMETER = 'update'
-
-  def extend_graph(self, graph, statements, format=rdf.Format.RDFXML):
-  #-------------------------------------------------------------------
-    self._request(self.ENDPOINTS[2], 'POST',
-                  body=urllib.urlencode({'data': statements,
-                                         'graph': str(graph),
-                                         'mime-type': rdf.Format.mimetype(format),
-                                        }),
-                  headers={'Content-type': 'application/x-www-form-urlencoded'})
-
-  def replace_graph(self, graph, statements, format=rdf.Format.RDFXML):
-  #--------------------------------------------------------------------
-    self._request(self.ENDPOINTS[2] + "?graph=%s" % graph, 'PUT',
-                  body=statements, headers={'Content-Type': rdf.Format.mimetype(format)})
-
-  def delete_graph(self, graph):
-  #-----------------------------
-    self._request(self.ENDPOINTS[2] + "?graph=%s" % graph, 'DELETE')
-
+def FourStore(SparqlUpdateStore):
+#================================
 
   def fulltext(self):
   #------------------
